#!/bin/bash
MODULE_ROOT_PATH=/lib/modules/`uname -r`
MODULE_INSTALL_PATH='kernel/drivers/dfrobot'
FILE_NAME_1='dfrobot_gt911'
FILE_NAME_2='panel-dfrobot-dsi'

function install_tools(){
    if apt list --installed 2>/dev/null | grep -q raspberrypi-kernel-headers && apt list --installed 2>/dev/null | grep -q make
    then
        return
    else 
        apt install -y make raspberrypi-kernel-headers
    fi
}

function set_config(){
    file_path="/boot/firmware/config.txt"

    if [ -f "$file_path" ]; then
        if grep -q $1=$2 /boot/firmware/config.txt
        then
            return
        else
            echo "$1=$2" >> /boot/firmware/config.txt
        fi
    else
        if grep -q $1=$2 /boot/config.txt
        then
            return
        else
            echo "$1=$2" >> /boot/config.txt
        fi
    fi

}
function remove_config(){
    file_path="/boot/firmware/config.txt"
    if [ -f "$file_path" ]; then
        if grep -q $1 /boot/firmware/config.txt
        then
            sed -i "/$1/d" /boot/firmware/config.txt
        fi
    else
        if grep -q $1 /boot/config.txt
        then
            sed -i "/$1/d" /boot/config.txt
        fi
    fi
}

function install_module() {
    if [ ! -d "$MODULE_ROOT_PATH/$MODULE_INSTALL_PATH" ]; then
        mkdir -p "$MODULE_ROOT_PATH/$MODULE_INSTALL_PATH"
    fi
    cp $1 "$MODULE_ROOT_PATH/$MODULE_INSTALL_PATH"
    cp $2 "$MODULE_ROOT_PATH/$MODULE_INSTALL_PATH"
    
    depmod
}


function remove_module(){
    if [ -f $MODULE_ROOT_PATH/$MODULE_INSTALL_PATH/$1 ]
    then
        rm $MODULE_ROOT_PATH/$MODULE_INSTALL_PATH/$1
    fi
    if [ -f $MODULE_ROOT_PATH/$MODULE_INSTALL_PATH/$2 ]
    then
        rm $MODULE_ROOT_PATH/$MODULE_INSTALL_PATH/$2
    fi
}

function setup(){
    set -e
    install_tools
    kernel_version=$(uname -r)
    if [[ $kernel_version == *"6.1"* ]]; then
        cd /opt/dfrobot_8_8_inch_dsi/src/6.1.y/
        make
        install_module $FILE_NAME_1.ko $FILE_NAME_2.ko
        dtc -@ -I dts -O dtb -o $1.dtbo $1.dts
        cp $1.dtbo /boot/overlays
        set_config dtoverlay $1
        make clean
    elif [[ $kernel_version == *"6.6"* ]]; then
        cd /opt/dfrobot_8_8_inch_dsi/src/6.6.y/
        make
        install_module $FILE_NAME_1.ko $FILE_NAME_2.ko
        dtc -@ -I dts -O dtb -o $1.dtbo $1.dts
        cp $1.dtbo /boot/overlays
        set_config dtoverlay $1
        make clean
    else
        echo "Not supported."
    fi

}

function remove(){
    set -e
    remove_module $FILE_NAME_1.ko $FILE_NAME_2.ko
    if [ -f /boot/overlays/$1.dtbo ]
    then
        rm /boot/overlays/$1.dtbo
    fi
    remove_config $1
}

function ask_reboot() {
    echo -e "Installation complete. Do you want to reboot? (y/n) \c"
    read user_input

    case "$user_input" in
        [yY])
            echo "Rebooting..."
            reboot
            ;;
        [nN])
            echo "User chose not to reboot."
            exit 0
            ;;
        *)
            echo "Invalid input. User canceled."
            exit 1
            ;;
    esac
}



function checkroot(){
    if [ `whoami` != "root" ]
    then
        echo "This tool requires root privileges.Try to run again with sudo..."
        exit 1
    fi
}

function main(){
    checkroot
    if [  $# -eq 1 ]
    then
        if [ $1 == "install" ]
        then
            setup  vc4-kms-dsi-dfrobot
        elif [ $1 == "remove" ]
        then
            remove vc4-kms-dsi-dfrobot
        else
            echo "Error: Script needs to provide parameters."
            echo "Usage: $0 install/remove"
            exit 1
        fi
    else
        echo "Error: Script needs to provide parameters."
        echo "Usage: $0 install/remove"
        exit 1
    fi
}
main $1
